package com.yeejoin.equipmanage.controller;

import java.util.Arrays;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import org.typroject.tyboot.core.foundation.enumeration.UserType;
import org.typroject.tyboot.core.restful.doc.TycloudOperation;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yeejoin.equipmanage.common.entity.ScrapDetail;
import com.yeejoin.equipmanage.common.utils.NameUtils;
import com.yeejoin.equipmanage.service.IScrapDetailService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 
 *
 * @author wujiang
 * @date 2020-07-07
 */
@RestController
@Api(tags = "报废详情Api")
@RequestMapping(value = "/scrap-detail", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
public class ScrapDetailController {

	@Autowired
	IScrapDetailService iScrapDetailService;

	/**
	 * 新增
	 * 
	 * @return
	 */
	@RequestMapping(value = "/save", method = RequestMethod.POST)
	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@ApiOperation(httpMethod = "POST", value = "新增", notes = "新增")
	public boolean saveScrapDetail(HttpServletRequest request, @RequestBody ScrapDetail scrapDetail) {
		return iScrapDetailService.save(scrapDetail);
	}

	/**
	 * 根据id删除
	 * 
	 * @param id
	 * @return
	 */
	@RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@ApiOperation(httpMethod = "DELETE", value = "根据id删除", notes = "根据id删除")
	public boolean deleteById(HttpServletRequest request, @PathVariable Long id) {
		return iScrapDetailService.removeById(id);
	}

	/**
	 * 修改
	 * 
	 * @return
	 */
	@RequestMapping(value = "/updateById", method = RequestMethod.PUT)
	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@ApiOperation(httpMethod = "PUT", value = "修改", notes = "修改")
	public boolean updateByIdScrapDetail(HttpServletRequest request, @RequestBody ScrapDetail scrapDetail) {
		return iScrapDetailService.updateById(scrapDetail);
	}

	/**
	 * 根据id查询
	 * 
	 * @param id
	 * @return
	 */
	@RequestMapping(value = "/{id}", method = RequestMethod.GET)
	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@ApiOperation(httpMethod = "GET", value = "根据id查询", notes = "根据id查询")
	public ScrapDetail selectById(HttpServletRequest request, @PathVariable Long id) {
		return iScrapDetailService.getOneById(id);
	}

	/**
	 * 列表分页查询
	 * 
	 * @return
	 */
	@PostMapping(value = "/list")
	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@ApiOperation(httpMethod = "POST", value = "列表分页查询", notes = "列表分页查询")
	public IPage<ScrapDetail> listPage(String pageNum, String pageSize, Long scrapId) {
		ScrapDetail scrapDetail = new ScrapDetail();
		scrapDetail.setScrapId(scrapId);
		Page<ScrapDetail> pageBean;

		QueryWrapper<ScrapDetail> scrapDetailQueryWrapper = new QueryWrapper<>();
		Class<? extends ScrapDetail> aClass = scrapDetail.getClass();
		Arrays.stream(aClass.getDeclaredFields()).forEach(field -> {
			try {
				field.setAccessible(true);
				Object o = field.get(scrapDetail);
				if (o != null) {
					Class<?> type = field.getType();
					String name = NameUtils.camel2Underline(field.getName());
					if (type.equals(Integer.class)) {
						Integer fileValue = (Integer) field.get(scrapDetail);
						scrapDetailQueryWrapper.eq(name, fileValue);
					} else if (type.equals(Long.class)) {
						Long fileValue = (Long) field.get(scrapDetail);
						scrapDetailQueryWrapper.eq(name, fileValue);
					} else if (type.equals(String.class)) {
						String fileValue = (String) field.get(scrapDetail);
						scrapDetailQueryWrapper.eq(name, fileValue);
					} else {
						String fileValue = (String) field.get(scrapDetail);
						scrapDetailQueryWrapper.eq(name, fileValue);
					}
				}
			} catch (Exception e) {
			}
		});
		IPage<ScrapDetail> page;
		if (StringUtils.isBlank(pageNum) || StringUtils.isBlank(pageSize)) {
			pageBean = new Page<>(0, Long.MAX_VALUE);
		} else {
			pageBean = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
		}
		page = iScrapDetailService.pageList(pageBean, scrapDetailQueryWrapper);
		return page;
	}
}
