package com.yeejoin.precontrol.controller;
import java.util.Arrays;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yeejoin.precontrol.common.entity.TaskRiskControlPrePerson;
import com.yeejoin.precontrol.common.service.ITaskRiskControlPrePersonService;
import com.yeejoin.precontrol.common.utils.NameUtils;
import com.yeejoin.precontrol.common.vo.TaskRiskControlPrePersonVo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;


/**
 * 风险管控预选人员表
 *
 * @author duanwei
 * @date 2021-01-28
 */
@RestController
@Api(tags = "风险管控预选人员表Api")
@RequestMapping(value = "/task-risk-control-pre-person", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
public class TaskRiskControlPrePersonController {

  @Autowired
  ITaskRiskControlPrePersonService iTaskRiskControlPrePersonService;
	/**
	 * 文件服务器地址
	 */
	@Value("${fileserver.domain}")
	String fileServerUrl;


    /**
    * 新增风险管控预选人员表
    * @return
    */
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    @ApiOperation(httpMethod = "POST", value = "新增风险管控预选人员表", notes = "新增风险管控预选人员表")
    public boolean saveTaskRiskControlPrePerson(HttpServletRequest request, @RequestBody TaskRiskControlPrePerson taskRiskControlPrePerson){
    return iTaskRiskControlPrePersonService.save(taskRiskControlPrePerson);
    }

    /**
    * 根据id删除
    * @param id
    * @return
    */
    @RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
    @ApiOperation(httpMethod = "DELETE", value = "根据id删除", notes = "根据id删除")
    public boolean deleteById(HttpServletRequest request, @PathVariable Long id){
    return iTaskRiskControlPrePersonService.removeById(id);
    }




    /**
    * 修改风险管控预选人员表
    * @return
    */
    @RequestMapping(value = "/updateById", method = RequestMethod.PUT)
    @ApiOperation(httpMethod = "PUT", value = "修改风险管控预选人员表", notes = "修改风险管控预选人员表")
    public boolean updateByIdTaskRiskControlPrePerson(HttpServletRequest request, @RequestBody TaskRiskControlPrePerson taskRiskControlPrePerson){
    return iTaskRiskControlPrePersonService.updateById(taskRiskControlPrePerson);
    }



    /**
   * 根据id查询
   * @param id
   * @return
   */
   @RequestMapping(value = "/{id}", method = RequestMethod.GET)
   @ApiOperation(httpMethod = "GET", value = "根据id查询", notes = "根据id查询")
   public TaskRiskControlPrePerson selectById(HttpServletRequest request, @PathVariable Long id){
      return iTaskRiskControlPrePersonService.getById(id);
   }



  /**
  * 列表分页查询
  * @return
  */
  @RequestMapping(value = "/list", method = RequestMethod.GET)
  @ApiOperation(httpMethod = "GET", value = "列表分页查询", notes = "列表分页查询")
  public IPage<TaskRiskControlPrePersonVo> listPage(String pageNum,String pageSize,
    TaskRiskControlPrePerson taskRiskControlPrePerson){

    Page<TaskRiskControlPrePerson> pageBean;
    QueryWrapper<TaskRiskControlPrePerson> taskRiskControlPrePersonQueryWrapper = new QueryWrapper<>();
    Class<? extends TaskRiskControlPrePerson> aClass = taskRiskControlPrePerson.getClass();
    Arrays.stream(aClass.getDeclaredFields()).forEach(field -> {
        try {
            field.setAccessible(true);
            Object o = field.get(taskRiskControlPrePerson);
            if (o != null) {
                Class<?> type = field.getType();
                String name = NameUtils.camel2Underline(field.getName());
                if (type.equals(Integer.class)) {
                Integer fileValue = (Integer) field.get(taskRiskControlPrePerson);
                taskRiskControlPrePersonQueryWrapper.eq(name, fileValue);
                } else if (type.equals(Long.class)) {
                Long fileValue = (Long) field.get(taskRiskControlPrePerson);
                taskRiskControlPrePersonQueryWrapper.eq(name, fileValue);
                } else if (type.equals(String.class)) {
                String fileValue = (String) field.get(taskRiskControlPrePerson);
                taskRiskControlPrePersonQueryWrapper.eq(name, fileValue);
                } else {
                String fileValue = (String) field.get(taskRiskControlPrePerson);
                taskRiskControlPrePersonQueryWrapper.eq(name, fileValue);
            }
          }
        }catch (Exception e) {
        }
    });
//    IPage<TaskRiskControlPrePerson> page;
//    if (StringUtils.isBlank(pageNum)  ||StringUtils.isBlank(pageSize)) {
//        pageBean = new Page<>(0, Long.MAX_VALUE);
//    }else{
//        pageBean = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
//    }
//    page = iTaskRiskControlPrePersonService.page(pageBean, taskRiskControlPrePersonQueryWrapper);
    List<TaskRiskControlPrePersonVo> list = iTaskRiskControlPrePersonService.listVo(taskRiskControlPrePerson);
    IPage<TaskRiskControlPrePersonVo> page1 = new Page<>(0, Long.MAX_VALUE);
    list.forEach(i->i.setHeadPhoto(fileServerUrl+i.getHeadPhoto()));
    page1.setRecords(list);
    return page1;
  }
}


